using System;
using System.Windows;
using GroupLab.Networking;
using System.Drawing;

namespace SideItem
{
	/// <summary>
	/// A nice name for the item to be used in menus and titles.
	/// </summary>
	public class ItemNameAttribute : System.Attribute
	{
		string name;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="name">Name of the item</param>
		public ItemNameAttribute(string name)
		{
			this.name = name;
		}

		/// <summary>
		/// Gets the item name.
		/// </summary>
		public string Name
		{
			get
			{
				return this.name;
			}
			set
			{
				this.name = value;
			}
		}
	}

	/// <summary>
	/// Summary description for Item.
	/// </summary>
	public interface IItem
	{
		/// <summary>
		/// Initialise the item.
		/// </summary>
		/// <param name="sd">The shared dictionary client instance for storing all the 
		/// shared data for this item.</param>
		/// <param name="path">The unique key to identify this item in the shared dictionary.</param>
		/// <param name="master">True if this item instance is the original one created.</param>
		/// <param name="currentUserKey">The key of the current user (the owner if master is true)</param>
		void StartItem(GroupLab.Networking.SharedDictionary sd, string path, bool master, string currentUserKey);

		/// <summary>
		/// Stop the item running.  Close down all running processes and clean up the 
		/// shared dictionary.
		/// </summary>
		void StopItem();

		/// <summary>
		/// Return a display for the tile view of this item at a particular awareness.
		/// </summary>
		/// <param name="focus">Value in the range 1-100. Indicates user's focus on the item.</param>
		/// <returns>A control that displays this item.</returns>
		System.Windows.Forms.Control GetTile(int focus);

		/// <summary>
		/// Return a display for the transient focus view of this item at a particular awareness.
		/// </summary>
		/// <param name="size">The maximum size that the application allows for this view.</param>
		/// <returns>A control that displays this item.</returns>
		System.Windows.Forms.Control GetTransientFocus(Size size);

		/// <summary>
		/// A handle for when the user wishes to separate this item from the application.  
		/// The result of calling this method should be a sparate form or application that allows
		/// the user full interaction.
		/// </summary>
		void SeparateItem();	

		/// <summary>
		/// Event indicating that the Item has changed it's internal state.
		/// </summary>
		event ChangeEventHandler ItemChanged;
	}
	/// <summary>
	/// Event arguments for a change event from the item.
	/// </summary>
	public class ChangeEventArgs : EventArgs
	{
		private int importance;
		/// <summary>
		/// Change event argument constructor.
		/// 
		/// Indicates the importance of the event. Importance is between 0 and 1.
		/// 1 indicates an important, major change.
		/// </summary>
		/// <param name="importance">Value between 0 and 1.</param>
		public ChangeEventArgs(int importance)
		{
			this.importance = importance;
		}

		/// <summary>
		/// Get the importance of this event.
		/// </summary>
		public int Importance
		{
			get
			{
				return this.importance;
			}
		}
	}

	/// <summary>
	/// Event Handler for change events to the item.
	/// </summary>
	public delegate void ChangeEventHandler(object sender, ChangeEventArgs e);
}
